package com.sksamuel.scoot.rest.client

import com.sksamuel.scoot.rest.{Logging, CrudRestMethods}
import javax.ws.rs.core.MediaType
import com.sksamuel.scoot.soa.Page
import com.sun.jersey.api.client.GenericType

/** @author Stephen Samuel */
abstract class CrudRestClient[T: Manifest, ID] extends CrudRestMethods[T, ID] with Logging {

    val resourceUrl: String
    val client: RestClient
    val mediaType: MediaType = MediaType.APPLICATION_JSON_TYPE

    def delete(id: ID): Int = client.resource(resourceUrl + "/" + id).delete.status

    /**
     * Returns the resource for the given id
     *
     */
    def get(id: ID): Option[T] = client.resource(resourceUrl + "/" + id).get.unmarshall[T]

    /**
     * Saves the given instance and returns the updated instance.
     */
    def save(t: T): Option[T] = client.resource(resourceUrl).post(t, mediaType).unmarshall[T]
}

trait SearchClient[T, ST] {

    val resourceUrl: String
    type PageType <: GenericType[Page[T]]
    val pageType: PageType
    val client: RestClient

    def search(search: ST): Option[Page[T]] = {
        client
          .resource(resourceUrl + "/search")
          .post(search, MediaType.APPLICATION_JSON_TYPE)
          .unmarshall(pageType)
    }

}