package com.sksamuel.scoot.rest.jersey.cache

import com.sun.jersey.spi.container._
import com.sun.jersey.api.model.AbstractMethod
import java.util.Collections
import javax.ws.rs.core.HttpHeaders
import java.util

/** @author Stephen Samuel */
class CacheFilterFactory extends ResourceFilterFactory {

    val NO_CACHE_FILTER = Collections.singletonList[ResourceFilter](new CacheResponseFilter("no-cache"))

    override def create(am: AbstractMethod) = {
        val cch = am.getAnnotation(classOf[CacheControl])
        if (cch == null) {
            val nocache = am.getAnnotation(classOf[NoCache])
            if (nocache == null) {
                new util.ArrayList[ResourceFilter]()
            } else {
                NO_CACHE_FILTER
            }
        } else {
            Collections.singletonList[ResourceFilter](new CacheResponseFilter(cch.value()))
        }
    }

    class CacheResponseFilter(headerValue: String) extends ResourceFilter {

        override def getRequestFilter = null

        override def getResponseFilter = {
            new ContainerResponseFilter() {

                override def filter(request: ContainerRequest, response: ContainerResponse) = {
                    // attache Cache Control header to each response based on the annotation value
                    response.getHttpHeaders.putSingle(HttpHeaders.CACHE_CONTROL, headerValue)
                    response
                }
            }
        }

    }
}
